package net.locusworks.s3sync.conf;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Properties;
import net.locusworks.logger.LogLevel;

public class ConfigurationManager {
  
  private static final String CONF_FILE= "s3.properties";
  private Properties conf;
  
  private static ConfigurationManager confManager;
  
  private ConfigurationManager() {
    init();
  }
  
  private void init() {
    try {
      Properties defaultProps = PropertiesManager.loadConfiguration(ConfigurationManager.class, CONF_FILE);
      Path confFile = Paths.get(CONF_FILE);
      
      if (Files.notExists(confFile)) {
        PropertiesManager.saveConfiguration(defaultProps, confFile, "AWS S3 Configuration");
        conf = defaultProps;
      } else {
        conf = PropertiesManager.loadConfiguration(confFile);
        PropertiesManager.addConfiguration(conf, defaultProps);
        PropertiesManager.removeConfiguration(conf, defaultProps);
        PropertiesManager.saveConfiguration(conf, confFile, "AWS S3 Configuration");
      }
      
    } catch (IOException e) {
      throw new RuntimeException(e);
    }
  }
  
  public String getRegion() {
    return conf.getProperty("region");
  }
  
  public String getBucketName() {
    return conf.getProperty("bucketName");
  }
  
  public Path getSyncFolder() {
    return Paths.get(conf.getProperty("syncFolder"));
  }
  
  public LogLevel getLogLevel() {
    return LogLevel.getEnum(conf.getProperty("logLevel"));
  }
  
  public static ConfigurationManager getInstance() {
    if (confManager == null) {
      confManager = new ConfigurationManager();
    }
    return confManager;
  }

}
